package Server;

import Shared.Board;
import Shared.Constants;
import Shared.Messages.ServerInitial;

import java.io.*;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Properties;

/**
 * Created by IntelliJ IDEA.
 * User: sehsanm
 * Date: Dec 13, 2004
 * Time: 10:42:02 PM
 * To change this template use Options | File Templates.
 */
public class Server {


    Board board;
    int timeOut;
    int boardSize;
    int listenPort;
    HashSet viewers = new HashSet();
    boolean redTurn;

    public Server(String fileName) throws IOException {
        Properties prop = new Properties();
        prop.load(new FileInputStream(fileName));

        timeOut = Integer.parseInt(prop.getProperty("Timeout"));
        boardSize = Integer.parseInt(prop.getProperty("Size"));
        listenPort = Integer.parseInt(prop.getProperty("Port"));
        board = new Board(boardSize);
    }

    public void runPlayers() {
        try {
            BufferedReader redInStream , blueInStream;
            OutputStreamWriter redWriter , blueWriter;
            Socket redPlayerSocket , bluePlayerSocket;
            ServerSocket server = new ServerSocket(listenPort);
            System.out.println("Listening");
            redPlayerSocket = server.accept();
            System.out.println("Red Accepted");
            bluePlayerSocket = server.accept();
            System.out.println("Blue Accepted");

            redPlayerSocket.setSoTimeout(timeOut);
            bluePlayerSocket.setSoTimeout(timeOut);

            redInStream = new BufferedReader(
                    new InputStreamReader(redPlayerSocket.getInputStream()));
            blueInStream = new BufferedReader(
                    new InputStreamReader(bluePlayerSocket.getInputStream()));

            redWriter = new OutputStreamWriter(redPlayerSocket.getOutputStream());
            blueWriter = new OutputStreamWriter(bluePlayerSocket.getOutputStream());


            redWriter.write((new ServerInitial(timeOut ,boardSize , Constants.RED)) + "\n");
            blueWriter.write((new ServerInitial(timeOut ,boardSize , Constants.BLUE)) + "\n");
            redWriter.flush();
            blueWriter.flush() ;
            redTurn = true;
            boolean gameTerminated = false ;
            while (!board.isFinished()) {
                BufferedReader turnReader;
                OutputStreamWriter turnWriter;
                if (redTurn) {
                    turnWriter = redWriter;
                    turnReader = redInStream;
                } else {
                    turnWriter = blueWriter;
                    turnReader = blueInStream;
                }

                turnWriter.write(sendBoard() + "\n");
                turnWriter.flush();
                String action;
                int cellOccupied = -1 ;
                try {

                    action = turnReader.readLine();
                    System.out.println("Action " + action);
                    int lineNo = Integer.parseInt(action) ;
                    if (redTurn)
                        cellOccupied =  board.mark(lineNo , Constants.RED);
                    else
                        cellOccupied = board.mark(lineNo , Constants.BLUE);
                } catch (SocketTimeoutException to) {
                    //TimeOut ;
                    System.out.println("Timeout happened. Terminating...");
                    gameTerminated = true ;
                    break ;
                }
                sendToAllViewers();
                if (cellOccupied == 0 ) {
                    redTurn = !redTurn ;
                }
                else if (cellOccupied == -1) {
                    //Illegal move
                    System.out.println("Illegal move. Terminating...");
                    gameTerminated = true ;
                    break ;
                }

            }
            if (gameTerminated)
            {
                System.out.println("Game Terminated!");
                if (redTurn)
                    System.out.println("Red player fouled!");
                else
                    System.out.println("Blue player fouled!");
            }
            else
            {

                System.out.println("Red got " + board.count(Constants.RED) + " cells");
                System.out.println("Blue got " + board.count(Constants.BLUE) + " cells");
            }
        } catch (IOException e) {
            e.printStackTrace();  //To change body of catch statement use Options | File Templates.
        }
    }

    public void acceptViewer() {
        try {
            ServerSocket ss = new ServerSocket(listenPort + 1);
            System.out.println("Listening for viewer");
            while (true) {
                Socket s = ss.accept() ;
                System.out.println("viewer accepted");
                OutputStreamWriter osw = new OutputStreamWriter (s.getOutputStream()) ;
                osw.write(boardSize + "\n");
                osw.flush();
                viewers.add(osw) ;
            }
        } catch (IOException e) {
            e.printStackTrace();  //To change body of catch statement use Options | File Templates.
        }
    }

    private String sendBoard() {

        String msg = board.toString() ;
        return msg;
    }


    private void sendToAllViewers() {
        for (Iterator iterator = viewers.iterator(); iterator.hasNext();) {
            OutputStreamWriter outputStreamWriter = (OutputStreamWriter) iterator.next();
            try {
                outputStreamWriter.write(board.toString() + "\n");
                outputStreamWriter.flush() ;
            } catch (IOException e) {
                e.printStackTrace();  //To change body of catch statement use Options | File Templates.
            }
        }
    }

    public void start() {
        Thread players = new Thread(new Runnable(){
            public void run() {
                runPlayers();
            }
        }) ;
        players.start();

        Thread viewers = new Thread(new Runnable() {
            public void run() {
                acceptViewer();
            }
        }) ;
        viewers.setDaemon(true);
        viewers.start();
    }

}
